package tree;

public class AbsoluteValueNode implements Node {

	private Node child;
	private Integer value;
	private TraversingStrategy strategy;
	private Node observer;

	public AbsoluteValueNode(Node child) {		
		this.child = child;
		this.value = child.getValue();
		this.strategy = new InorderTraversing();
	}

	public void traverse(Visitor v) {
		strategy.traverseAbsoluteValueNode(this, v);
	}
	
	public void visit(Visitor v) {
		if(v instanceof AbsoluteValueNodeVisitor)
			((AbsoluteValueNodeVisitor)v).visitAbsoluteValueNode(this);
		else
			throw new IncompatibleVisitorException();
	}
	
	public Node getChild() {
		return child;
	}

	public Integer getValue() {
		return value;
	}
	
	public void updateValue() {
		value = child.getValue();
		notifyObserver();
	}
	
	public void notifyObserver() {
		if(this.observer != null)
			observer.updateValue();
	}
	
	public void attachObserver(Node observer) {
		this.observer = observer;
	}
	
	public Node deattachObserver() {
		Node observer = this.observer; 
		this.observer = null;
		return observer; 
	}
	
	public void changeStrategy(TraversingStrategy strategy) {
		this.strategy = strategy;
		child.changeStrategy(strategy);
	}
	
	public TraversingStrategy getStrategy() {
		return strategy;
	}
	
}
