package tree;

public class BinaryOperatorNode implements Node {

	private Command command;
	private Node leftChild;
	private Node rightChild;
	private Integer value;
	private TraversingStrategy strategy;
	private Node observer;
	
	public BinaryOperatorNode(Command command, Node leftChild, Node rightChild) {
		this.command = command;
		this.leftChild = leftChild;
		this.rightChild = rightChild;
		this.value = command.execute(leftChild.getValue(), rightChild.getValue());
		this.strategy = new InorderTraversing();
	}
	
	public void traverse(Visitor v) {
		strategy.traverseBinaryOperatorNode(this, v);
	}
	
	public void visit(Visitor v) {
		if(v instanceof BinaryOperatorNodeVisitor)
			((BinaryOperatorNodeVisitor)v).visitBinaryOperatorNode(this);		
		else
			throw new IncompatibleVisitorException();
	}
	
	public Node getLeftChild() {
		return leftChild;
	}
	
	public Node getRightChild() {
		return rightChild;
	}
	
	public void changeStrategy(TraversingStrategy strategy) {
		this.strategy = strategy;
		leftChild.changeStrategy(strategy);
		rightChild.changeStrategy(strategy);
	}

	public TraversingStrategy getStrategy() {
		return strategy;
	}
	
	public Integer getValue() {
		return value;
	}
	
	public void updateValue() {
		value = command.execute(leftChild.getValue(), rightChild.getValue());
		notifyObserver();
	}
	
	public void notifyObserver() {
		if(observer != null)
			observer.updateValue();
	}
	
	public void attachObserver(Node observer) {		
		this.observer = observer;
	}
	
	public Node deattachObserver() {
		Node observer = this.observer; 
		this.observer = null;
		return observer; 
	}
	
	public Command getType() {
		return command;
	}
	
	public void changeType(Command command) {
		this.command = command;
		this.value = command.execute(leftChild.getValue(), rightChild.getValue());
		notifyObserver();
	}
}
