package tree;

public class ValueNode implements Node {

	private Integer value;
	private Node observer;
	private TraversingStrategy strategy;
	
	public ValueNode(Integer value) {
		this.value = value;
		this.strategy = new InorderTraversing();
	}
	
	public void traverse(Visitor v) {
		strategy.traverseValueNode(this, v);
	}

	public void visit(Visitor v) {
		if(v instanceof ValueNodeVisitor)
			((ValueNodeVisitor)v).visitValueNode(this);
		else
			throw new IncompatibleVisitorException();
	}
	
	public Integer getValue() {
		return value;
	}
	
	public void setValue(Integer value) {
		this.value = value;
		notifyObserver();
	}	
	
	public void updateValue() {
		//nothingh (this is for the Observer)
	}
	
	public void notifyObserver() {
		if(observer != null)
			observer.updateValue();
	}
	
	public void attachObserver(Node observer) {
		this.observer = observer;
	}
	
	public Node deattachObserver() {
		Node observer = this.observer; 
		this.observer = null;
		return observer; 
	}
	
	public void changeStrategy(TraversingStrategy strategy) {
		this.strategy = strategy;
	}
	
	public TraversingStrategy getStrategy() {
		return strategy;
	}
	
}
